@@ -143,7 +143,7 @@ EXPERIMENTAL: Interface subject to change."
                                sb-kernel:*current-internal-error-context*
                                (sb-sys:int-sap 0))
                            0))
-      ;; %INTERRUPT-THREAD requires that the interruptions lock be held by the caller.
+      ;; %INTERRUPT-THREAD requires that the storage lock be held by the caller.
       (sb-thread:with-deathlok (thread c-thread)
         (when (and (/= c-thread 0)
                    (zerop (sap-ref-8 (int-sap c-thread) (sprof-enable-byte))))
@@ -190,20 +190,13 @@ EXPERIMENTAL: Interface subject to change."
           ((eq code sb-fasl:*assembler-routines*)
            (values (sb-disassem::find-assembler-routine pc-int)
                    pc-int :asm-routine))
+          ;; This is mainly just defensive, e.g. if interrupted in code resulting
+          ;; from MAKE-BPT-LRA. There should otherwise not exist code lacking debug info
+          ((not (typep (sb-kernel:%code-debug-info code) 'sb-c::compiled-debug-info))
+           (values code pc-int nil))
           (t
-           (let* (;; Give up if we land in the 2 or 3 instructions of a
-                  ;; code component sans simple-fun that is not an asm routine.
-                  ;; While it's conceivable that this could be improved,
-                  ;; the problem will be different or nonexistent after
-                  ;; funcallable-instances each contain their own trampoline.
-                  #+immobile-code
-                  (di (unless (typep (sb-kernel:%code-debug-info code)
-                                     'sb-c::compiled-debug-info)
-                        (return-from debug-info
-                          (values code pc-int nil))))
-                  (pc-offset (sap- (int-sap pc-int) (sb-kernel:code-instructions code)))
+           (let* ((pc-offset (sap- (int-sap pc-int) (sb-kernel:code-instructions code)))
                   (df (sb-di::debug-fun-from-pc code pc-offset)))
-             #+immobile-code (declare (ignorable di))
              (cond ((typep df 'sb-di::bogus-debug-fun)
                     (values code pc-int nil))
                    (df
@@ -344,11 +337,11 @@ EXPERIMENTAL: Interface subject to change."
              ;; Ensure that the thread can't exit (which ensures that it can't free the sprof_sem
              ;; that might be needed by the C routine, depending on things), and also ensure
              ;; mutual exclusivity with other callers of this.
-             (sb-thread:with-deathlok (thread c-thread)
-               (unless (zerop c-thread)
-                 (let ((sap (alien-funcall acquire-data c-thread)))
-                   (unless (= (sap-int sap) 0)
-                     (process sap thread))))))
+             (let ((sap (sb-thread:with-deathlok (thread c-thread)
+                          (unless (zerop c-thread)
+                            (alien-funcall acquire-data c-thread)))))
+               (when (and sap (/= (sap-int sap) 0))
+                 (process sap thread))))
            nil)
          all-threads)))))
 
