@@ -50,7 +50,7 @@
 
 ;;; Used for interning parts of SLOT-NAME-LISTS, as part of
 ;;; PV-TABLE interning -- just to save space.
-(define-load-time-global *slot-name-lists* (make-hash-table :test 'equal))
+(define-load-time-global *slot-name-lists* (make-hashset 64 #'list-elts-eq #'sxhash))
 
 ;;; Used for interning PV-TABLES, keyed by the SLOT-NAME-LISTS
 ;;; used.
@@ -58,24 +58,28 @@
 
 ;;; ...and one lock to rule them. Lock because for certain (rare)
 ;;; cases this lock might be grabbed in the course of method dispatch
-;;; -- and mostly this is already under the *world-lock*
+;;; -- and mostly this is already under the **world-lock**
 (define-load-time-global *pv-lock*
   (sb-thread:make-mutex :name "pv table index lock"))
 
 (defun intern-pv-table (&key slot-name-lists)
   (flet ((intern-slot-names (slot-names)
-           (ensure-gethash slot-names *slot-name-lists* slot-names))
+           ;; Hashsets don't like NIL as a key because NIL is the GC-splatted marker
+           ;; (this isn't a weak hashset, but still, don't try to store NIL because
+           ;; it causes the set to size up on every alleged failure to find)
+           (if slot-names
+               (hashset-insert-if-absent *slot-name-lists* slot-names #'identity)))
          (%intern-pv-table (snl)
            (ensure-gethash
             snl *pv-tables*
             (make-pv-table :slot-name-lists snl
                            :pv-size (* 2 (reduce #'+ snl :key #'length))))))
-    (sb-thread:with-mutex (*pv-lock*)
+    (with-system-mutex (*pv-lock*)
       (%intern-pv-table (mapcar #'intern-slot-names slot-name-lists)))))
 
 (defun use-standard-slot-access-p (class slot-name type)
   (or (not (eq **boot-state** 'complete))
-      (and (standard-class-p class)
+      (and (std-class-p class)
            (let ((slotd (find-slot-definition class slot-name)))
              (and slotd
                   (slot-accessor-std-p slotd type))))))
@@ -88,7 +92,10 @@
              (and (slot-missing class object slot-name 'slot-boundp) t))
    :writer (lambda (new-value object)
              (slot-missing class object slot-name 'setf new-value)
-             new-value)))
+             new-value)
+   :makunbound (lambda (object)
+                 (slot-missing class object slot-name 'slot-makunbound)
+                 object)))
 
 (defun compute-pv (slot-name-lists wrappers)
   (let ((wrappers (ensure-list wrappers)))
@@ -127,9 +134,6 @@
             (unless (eq new-cache cache)
               (setf (pv-table-cache pv-table) new-cache))
             pv)))))
-
-(defun make-pv-type-declaration (var)
-  `(type simple-vector ,var))
 
 ;;; Sometimes we want to finalize if we can, but it's OK if
 ;;; we can't.
@@ -152,11 +156,9 @@
 
 (defun can-optimize-access (form required-parameters env)
   (destructuring-bind (op var-form slot-name-form &optional new-value) form
-    (let ((type (ecase op
-                  (slot-value 'reader)
-                  (set-slot-value 'writer)
-                  (slot-boundp 'boundp)))
-          (var (extract-the var-form))
+    (declare (ignore op))
+    (aver (constantp slot-name-form env))
+    (let ((var (extract-the var-form))
           (slot-name (constant-form-value slot-name-form env)))
       (when (and (symbolp var) (not (var-special-p var env)))
         (let* ((rebound? (caddr (var-declaration '%variable-rebinding var env)))
@@ -180,13 +182,9 @@
                           class form))
                        (setf class nil))))
               (when (and class-name (not (eq class-name t)))
-                (when (not (and class
-                                (memq *the-class-structure-object*
-                                      (class-precedence-list class))))
-                  (aver type)
-                  (values (cons parameter-or-nil (or class class-name))
-                          slot-name
-                          new-value))))))))))
+                (values (cons parameter-or-nil (or class class-name))
+                        slot-name
+                        new-value)))))))))
 
 ;;; Check whether the binding of the named variable is modified in the
 ;;; method body.
@@ -267,6 +265,22 @@
       `(accessor-slot-boundp ,@(cdr form))
       optimized-form))
 
+(defun optimize-slot-makunbound (form slots required-parameters env)
+  (multiple-value-bind (sparameter slot-name)
+      (can-optimize-access form required-parameters env)
+    (if sparameter
+        (let ((optimized-form
+                (optimize-instance-access slots :makunbound sparameter
+                                          slot-name nil)))
+          `(optimized-slot-makunbound ,form ,(car sparameter) ,optimized-form))
+        `(accessor-slot-makunbound ,@(cdr form)))))
+
+(defmacro optimized-slot-makunbound (form parameter-name optimized-form
+                                     &environment env)
+  (if (parameter-modified-p parameter-name env)
+      `(accessor-slot-makunbound ,@(cdr form))
+      optimized-form))
+
 ;;; The SLOTS argument is an alist, the CAR of each entry is the name
 ;;; of a required parameter to the function. The alist is in order, so
 ;;; the position of an entry in the alist corresponds to the
@@ -274,20 +288,27 @@
 (defun optimize-instance-access (slots read/write sparameter slot-name
                                  new-value &optional safep)
   (let ((class (if (consp sparameter) (cdr sparameter) *the-class-t*))
-        (parameter (if (consp sparameter) (car sparameter) sparameter)))
+        (parameter (if (consp sparameter) (car sparameter) sparameter))
+        slotd)
     (if (and (eq **boot-state** 'complete)
              (classp class)
-             (memq *the-class-structure-object* (class-precedence-list class)))
-        (let ((slotd (find-slot-definition class slot-name)))
-          (ecase read/write
-            (:read
-             `(,(slot-definition-defstruct-accessor-symbol slotd) ,parameter))
-            (:write
-             `(setf (,(slot-definition-defstruct-accessor-symbol slotd)
-                     ,parameter)
-                    ,new-value))
-            (:boundp
-             t)))
+             (typep (setq slotd (find-slot-definition class slot-name))
+                    'structure-effective-slot-definition))
+        (ecase read/write
+          (:read
+           `(,(slot-definition-defstruct-accessor-symbol slotd) ,parameter))
+          (:write
+           `(setf (,(slot-definition-defstruct-accessor-symbol slotd)
+                   ,parameter)
+                  ,new-value))
+          (:boundp
+           (if (slot-definition-always-bound-p slotd)
+               t
+               ;; no need for speed for BOUNDP on structure slots
+               `(accessor-slot-boundp ,parameter ,slot-name)))
+          (:makunbound
+           ;; no need for speed for MAKUNBOUND on structure slots
+           `(accessor-slot-makunbound ,parameter ,slot-name)))
         (let* ((parameter-entry (assq parameter slots))
                (slot-entry      (assq slot-name (cdr parameter-entry)))
                (position (posq parameter-entry slots))
@@ -308,7 +329,10 @@
                                 ',slot-name ',class .new-value. ,safep)))
             (:boundp
              `(instance-boundp ,pv-offset-form ,parameter ,position
-                               ',slot-name ',class)))))))
+                               ',slot-name ',class))
+            (:makunbound
+             `(instance-makunbound ,pv-offset-form ,parameter ,position
+                                   ',slot-name ',class)))))))
 
 (define-walker-template pv-offset) ; These forms get munged by mutate slots.
 (defmacro pv-offset (arg) arg)
@@ -338,10 +362,9 @@
   (if (eq **boot-state** 'complete)
       (let (slotd)
         (cond ((or
-                ;; Conditions, structures, and classes for which FIND-CLASS
+                ;; Conditions, and classes for which FIND-CLASS
                 ;; doesn't return them yet.
-                ;; FIXME: surely we can get faster accesses for structures?
-                (not (standard-class-p class))
+                (not (std-class-p class))
                 ;; Should not happen... (FIXME: assert instead?)
                 (eq class *the-class-t*)
                 (not (class-finalized-p class))
@@ -382,23 +405,11 @@
        (let ((,index (svref ,pv ,pv-offset))
              (,slots (truly-the simple-vector ,slots)))
          (setq ,value (typecase ,index
-                        ;; FIXME: the line marked by KLUDGE below (and
-                        ;; the analogous spot in
-                        ;; INSTANCE-WRITE-STANDARD) is there purely to
-                        ;; suppress a type mismatch warning that
-                        ;; propagates through to user code.
-                        ;; Presumably SLOTS at this point can never
-                        ;; actually be NIL, but the compiler seems to
-                        ;; think it could, so we put this here to shut
-                        ;; it up.  (see also mail Rudi Schlatte
-                        ;; sbcl-devel 2003-09-21) -- CSR, 2003-11-30
                         ,@(when (or (null kind) (eq kind :instance))
-                                `((fixnum
-                                   (clos-slots-ref ,slots ,index))))
+                            `((fixnum (clos-slots-ref ,slots ,index))))
                         ,@(when (or (null kind) (eq kind :class))
-                                `((cons (cdr ,index))))
-                        (t
-                         +slot-unbound+)))
+                            `((cons (cdr ,index))))
+                        (t +slot-unbound+)))
          (if (unbound-marker-p ,value)
              ,default
              ,value)))))
@@ -448,14 +459,13 @@
               new-value)))
     `(locally (declare #.*optimize-speed*)
        (let ((.good-new-value. ,new-value-form)
-             (,index (svref ,pv ,pv-offset)))
+             (,index (svref ,pv ,pv-offset))
+             (,slots (truly-the simple-vector ,slots)))
          (typecase ,index
            ,@(when (or (null kind) (eq kind :instance))
-                   `((fixnum (and ,slots
-                                  (setf (clos-slots-ref ,slots ,index)
-                                        .good-new-value.)))))
+               `((fixnum (setf (clos-slots-ref ,slots ,index) .good-new-value.))))
            ,@(when (or (null kind) (eq kind :class))
-                   `((cons (setf (cdr ,index) .good-new-value.))))
+               `((cons (setf (cdr ,index) .good-new-value.))))
            (t ,default))))))
 
 (defmacro instance-write-custom (pv pv-offset parameter new-value)
@@ -485,20 +495,58 @@
     (error "illegal kind argument to ~S: ~S" 'instance-boundp-standard kind))
   (let* ((index (gensym)))
     `(locally (declare #.*optimize-speed*)
-       (let ((,index (svref ,pv ,pv-offset)))
+       (let ((,index (svref ,pv ,pv-offset))
+             (,slots (truly-the simple-vector ,slots)))
          (typecase ,index
            ,@(when (or (null kind) (eq kind :instance))
-                   `((fixnum (not (and ,slots
-                                       (unbound-marker-p
-                                        (clos-slots-ref ,slots ,index)))))))
+               `((fixnum (not (unbound-marker-p (clos-slots-ref ,slots ,index))))))
            ,@(when (or (null kind) (eq kind :class))
-                   `((cons (not (unbound-marker-p (cdr ,index))))))
+               `((cons (not (unbound-marker-p (cdr ,index))))))
            (t ,default))))))
 
 (defmacro instance-boundp-custom (pv pv-offset parameter)
   `(locally (declare #.*optimize-speed*)
      (funcall (slot-info-boundp (svref ,pv (1+ ,pv-offset))) ,parameter)))
 
+;;;; SLOT-MAKUNBOUND
+
+(defmacro instance-makunbound (pv-offset parameter position slot-name class)
+  (ecase (slot-access-strategy (constant-value-or-nil class)
+                               (constant-value-or-nil slot-name)
+                               'makunbound)
+    (:standard
+     `(instance-makunbound-standard
+       .pv. ,(slot-vector-symbol position)
+       ,pv-offset (accessor-slot-makunbound ,parameter ,slot-name)
+       ,parameter
+       ,(if (generate-fast-class-slot-access-p class slot-name)
+            :class :instance)))
+    (:custom
+     `(instance-makunbound-custom .pv. ,pv-offset ,parameter))
+    (:accessor
+     `(accessor-slot-makunbound ,parameter ,slot-name))))
+
+(defmacro instance-makunbound-standard (pv slots pv-offset default parameter
+                                        &optional kind)
+  (unless (member kind '(nil :instance :class))
+    (error "illegal kind argument to ~S: ~S" 'instance-makunbound-standard kind))
+  (let* ((index (gensym)))
+    `(locally (declare #.*optimize-speed*)
+       (let ((,index (svref ,pv ,pv-offset))
+             (,slots (truly-the simple-vector ,slots)))
+         (typecase ,index
+           ,@(when (or (null kind) (eq kind :instance))
+               `((fixnum (setf (clos-slots-ref ,slots ,index) +slot-unbound+)
+                         ,parameter)))
+           ,@(when (or (null kind) (eq kind :class))
+               `((cons (setf (cdr ,index) +slot-unbound+)
+                       ,parameter)))
+           (t ,default))))))
+
+(defmacro instance-makunbound-custom (pv pv-offset parameter)
+  `(locally (declare #.*optimize-speed*)
+     (funcall (slot-info-makunbound (svref ,pv (1+ ,pv-offset))) ,parameter)))
+
 ;;; This magic function has quite a job to do indeed.
 ;;;
 ;;; The careful reader will recall that <slots> contains all of the
@@ -592,7 +640,7 @@
          ,@forms)
       `(let* ((.pv-table. ,pv-table-form)
               (.pv. (pv-table-lookup-pv-args .pv-table. ,@pv-parameters)))
-        (declare ,(make-pv-type-declaration '.pv.))
+        (declare (simple-vector .pv.))
         ,@forms)))
 
 (defun split-declarations (body args req-args cnm-p parameters-setqd)
